import 'package:sql_query/builder/query.dart';

import '../database_config.dart';
import '../database_model.dart';

class DB {
  static List<Map<String, String>>? listTableQuery;
  static List<Map<String, String>>? listForeignKeyQuery;
  static Query table(String table) {
    return Query(
      table: table,
    );
  }

  static String _tableBuilder(TableDatabase data) {
    String insert = 'CREATE TABLE ${data.tableName} (';
    int total = 0;
    for (final tc in data.column) {
      ++total;
      String prop = '';

      if (tc.typeData == DataType.text) {
        prop += ' TEXT';
      } else if (tc.typeData == DataType.integer) {
        // if (tc.typeData == DataType.bool) {
        //   _boolData.add(tc.name);
        // }
        prop += ' INTEGER';
      } else if (tc.typeData == DataType.double) {
        prop += ' REAL';
      }

      if (tc.primaryKey) {
        prop += ' PRIMARY KEY';
      }

      if (tc.autoIncrement) {
        prop += ' AUTOINCREMENT';
      }

      if (!tc.isNull) {
        prop += ' NOT NULL';
      }

      if (tc.defaultValue != null) {
        prop += ' DEFAULT ${tc.typeData == DataType.text ? '"' : ''}${tc.defaultValue}${tc.typeData == DataType.text ? '"' : ''}';
      }

      insert += '${tc.name} ' + prop;
      if (data.column.length > 1 && total != data.column.length) {
        insert += ', ';
      }
    }
    if (data.foreignKey != null) {
      insert += ' , ';
      insert += ' FOREIGN KEY(${data.foreignKey?.baseColumn}) REFERENCES ${data.foreignKey?.referenceTable}(${data.foreignKey?.referenceColumn}) ';
      if (data.foreignKey!.deleteAction != null) {
        insert += ForeignKey.getAction(data.foreignKey!.deleteAction!, 'DELETE');
      }

      if (data.foreignKey!.updateAction != null) {
        insert += ForeignKey.getAction(data.foreignKey!.updateAction!, 'UPDATE');
      }
    }
    insert += ')';
    return insert;
  }

  static sqlQueryTables() {
    return TableDatabase(
      column: [
        ColumnDatabase(name: 'id', typeData: DataType.integer, primaryKey: true, autoIncrement: true),
        ColumnDatabase(name: 'name_table', typeData: DataType.text),
        ColumnDatabase(name: 'name_column', typeData: DataType.text),
        ColumnDatabase(name: 'is_primary', typeData: DataType.integer),
        ColumnDatabase(name: 'is_ai', typeData: DataType.integer),
        ColumnDatabase(name: 'config', typeData: DataType.text),
      ],
      tableName: '_sql_query_tables',
    );
  }

  static foreignKeyTables() {
    return TableDatabase(
      column: [
        ColumnDatabase(name: 'id', typeData: DataType.integer, primaryKey: true, autoIncrement: true),
        ColumnDatabase(name: 'name_table', typeData: DataType.text),
        ColumnDatabase(name: 'name_column', typeData: DataType.text),
        ColumnDatabase(name: 'name_table_target', typeData: DataType.text),
        ColumnDatabase(name: 'name_column_target', typeData: DataType.text),
        ColumnDatabase(name: 'on_delete', typeData: DataType.text),
        ColumnDatabase(name: 'on_update', typeData: DataType.text),
      ],
      tableName: '_sql_query_foreign_key',
    );
  }

  static Map<String, String> buildToQuery({nameTable, nameColumn, isPirmary, isAI, config}) {
    return {
      'name_table': nameTable,
      'name_column': nameColumn,
      'is_primary': isPirmary,
      'is_ai': isAI,
      'config': config,
    };
  }

  static Map<String, String> buildForeignKeyToQuery({nameTable, nameColumn, nameTableTarget, nameColumnTarget, delete, update}) {
    return {
      'name_table': nameTable,
      'name_column': nameColumn,
      'name_table_target': nameTableTarget,
      'name_column_target': nameColumnTarget,
      'on_delete': delete,
      'on_update': update,
    };
  }

  static init({required List<TableDatabase> tableList, String? databaseName, String? customPath}) async {
    List<String> insertList = [_tableBuilder(sqlQueryTables()), _tableBuilder(foreignKeyTables())];
    List<TableDatabase> foreignKey = [];
    listForeignKeyQuery = [];
    listTableQuery = [];
    for (final tl in tableList) {
      insertList.add(_tableBuilder(tl));
      for (final col in tl.column) {
        listTableQuery!.add(
          buildToQuery(
            nameTable: tl.tableName,
            nameColumn: col.name,
            isAI: col.autoIncrement ? "1" : '0',
            isPirmary: col.primaryKey ? '1' : '0',
            config: '${col.typeData}|${col.isNull}|${col.defaultValue}',
          ),
        );
      }
      if (tl.foreignKey != null) {
        listForeignKeyQuery!.add(
          buildForeignKeyToQuery(
            nameColumn: tl.foreignKey?.baseColumn,
            nameColumnTarget: tl.foreignKey?.referenceColumn,
            nameTable: tl.tableName,
            nameTableTarget: tl.foreignKey?.referenceTable,
            delete: tl.foreignKey?.deleteAction.toString(),
            update: tl.foreignKey?.updateAction.toString(),
          ),
        );
      }
    }

    for (final tl in foreignKey) {
      insertList.add(_tableBuilder(tl));
    }

    await DatabaseConfig.init(
      tables: insertList,
      dbName: databaseName,
      rawTables: tableList,
      customPath: customPath,
    );
  }

  static alterTable(String rawSqfliteQuery) async {
    await DatabaseConfig.database!.transaction((txn) async {
      await txn.execute(rawSqfliteQuery);
    });
  }

  static deleteDB() {
    DatabaseConfig.dbDelete();
  }

  static closeDB() {
    DatabaseConfig.dbClose();
  }
}
