enum DataType { text, integer, double }

enum ForeignKeyAction { setnull, setDefault, restrict, setnoAction, setCascade }

class ForeignKey {
  String referenceTable, referenceColumn, baseColumn;
  String? baseTable;
  ForeignKeyAction? updateAction;
  ForeignKeyAction? deleteAction;

  static parseAction(string) {
    if (string == 'ForeignKeyAction.setnull') {
      return ForeignKeyAction.setnull;
    } else if (string == 'ForeignKeyAction.setDefault') {
      return ForeignKeyAction.setDefault;
    } else if (string == 'ForeignKeyAction.restrict') {
      return ForeignKeyAction.restrict;
    } else if (string == 'ForeignKeyAction.setnoAction') {
      return ForeignKeyAction.setnoAction;
    } else if (string == 'ForeignKeyAction.setCascade') {
      return ForeignKeyAction.setCascade;
    }
  }

  static String getAction(
    ForeignKeyAction action,

    /// `Update` , `Delete`
    String type,
  ) {
    String query = '';
    if (action == ForeignKeyAction.setnull) {
      query += ' ON ${type.toUpperCase()} SET NULL ';
    } else if (action == ForeignKeyAction.restrict) {
      query += ' ON ${type.toUpperCase()} RESTRICT ';
    } else if (action == ForeignKeyAction.setCascade) {
      query += ' ON ${type.toUpperCase()} CASCADE ';
    } else if (action == ForeignKeyAction.setDefault) {
      query += ' ON ${type.toUpperCase()} SET DEFAULT ';
    } else if (action == ForeignKeyAction.setnoAction) {
      query += ' ON ${type.toUpperCase()} NO ACTION ';
    }
    return query;
  }

  ForeignKey({required this.baseColumn, required this.referenceTable, required this.referenceColumn, this.updateAction = ForeignKeyAction.restrict, this.deleteAction = ForeignKeyAction.restrict, this.baseTable});
}

class TableDatabase {
  String tableName;
  List<ColumnDatabase> column;
  ForeignKey? foreignKey;

  TableDatabase({required this.column, required this.tableName, this.foreignKey});
}

class ColumnDatabase {
  bool primaryKey, autoIncrement, isNull;
  String name;
  String? defaultValue;
  DataType typeData;

  static typeDataParse(data) {
    if (data == 'DataType.integer') {
      return DataType.integer;
    } else if (data == 'DataType.text') {
      return DataType.text;
    } else if (data == 'DataType.double') {
      return DataType.double;
    }
  }

  ColumnDatabase({this.autoIncrement = false, this.isNull = false, required this.name, required this.typeData, this.primaryKey = false, this.defaultValue});
}
