import 'package:flutter/material.dart';
import 'package:sql_query/viewer/controller/sql_viewer_controller.dart';
import 'package:sql_query/viewer/part/form.dart';
import 'package:sql_query/viewer/part/select.dart';

class QueryController {
  static late MaterialXFormController limit;
  static late MaterialXFormController search;
  static late BuildContext _currentContext;
  static SelectData? selectedFilterColumn;

  static widgetSelect(context, {buttonName}) {
    return GestureDetector(
      onTap: () {
        selectColumn(context);
      },
      child: Padding(
        padding: const EdgeInsets.symmetric(vertical: 2.0),
        child: Container(
          padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4),
            color: Colors.blue,
          ),
          child: Text(
            buttonName ?? 'select',
            style: const TextStyle(color: Colors.white, fontSize: 11),
          ),
        ),
      ),
    );
  }

  static init(context) {
    _currentContext = context;
    limit = MaterialXFormController(controller: TextEditingController(text: '50'));
    search = MaterialXFormController(suffix: widgetSelect(context));
  }

  static selectColumn(context) {
    Select.single(
      title: 'Select Filter Target Column',
      context: context,
      data: SQL.selectColumn,
    ).then((value) {
      if (value != null) {
        selectedFilterColumn = value;
        search.suffix = widgetSelect(context, buttonName: value.title);
        SQL.dataTable.refresh((listener) => null);
      }
    });
  }

  static clear() {
    limit.controller?.text = '50';
    search.controller?.text = '';
    selectedFilterColumn = null;
    search.suffix = widgetSelect(_currentContext);
  }

  static dispose() {
    limit.controller?.dispose();
    search.controller?.dispose();
  }
}
