import 'package:flutter/material.dart';
import 'package:flutter_feather_icons/flutter_feather_icons.dart';
import 'package:sql_query/viewer/controller/sql_viewer_controller.dart';
import 'package:sql_query/viewer/model/auto_model.dart';

import '../../database_config.dart';
import 'button.dart';

class Sidebar extends StatelessWidget {
  const Sidebar({super.key});

  @override
  Widget build(BuildContext context) {
    return Drawer(
      child: StatefulBuilder(builder: (context, s) {
        return Column(
          children: [
            const SizedBox(
              height: 40,
            ),
            Padding(
              padding: const EdgeInsets.only(right: 20.0),
              child: Container(
                padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                decoration: const BoxDecoration(
                  color: Color(0xff2463C2),
                  borderRadius: BorderRadius.only(
                    topRight: Radius.circular(20),
                    bottomRight: Radius.circular(20),
                  ),
                ),
                child: Row(
                  children: [
                    const Icon(
                      FeatherIcons.database,
                      color: Colors.white,
                      size: 40,
                    ),
                    const SizedBox(
                      width: 12,
                    ),
                    Expanded(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            DatabaseConfig.path.split('/').last,
                            style: const TextStyle(
                              color: Colors.white,
                              fontSize: 18,
                            ),
                          ),
                          Text(
                            '${DatabaseConfig.size / 1024} KB',
                            style: const TextStyle(
                              color: Colors.white,
                              fontSize: 11,
                            ),
                          ),
                        ],
                      ),
                    ),
                    const SizedBox(
                      width: 12,
                    ),
                    NoSplashButton(
                      onTap: () async {
                        await SQL.refresh();
                        s(() {});
                      },
                      child: const Icon(
                        Icons.replay_outlined,
                        color: Colors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
            const SizedBox(
              height: 12,
            ),
            Expanded(
              child: SingleChildScrollView(
                child: Fresher(
                    listener: SQL.dataTable,
                    builder: (value) {
                      return ListView.builder(
                          padding: EdgeInsets.zero,
                          physics: const NeverScrollableScrollPhysics(),
                          shrinkWrap: true,
                          itemCount: value.length,
                          itemBuilder: (context, i) {
                            final data = value[i];
                            return NoSplashButton(
                              onTap: () {
                                SQL.selectTable(data);
                                Navigator.pop(context);
                              },
                              child: Padding(
                                padding: const EdgeInsets.only(bottom: 10.0),
                                child: IntrinsicHeight(
                                  child: Row(
                                    children: [
                                      SQL.selectedTable == data
                                          ? Padding(
                                              padding: const EdgeInsets.only(right: 8),
                                              child: Container(
                                                color: const Color(0xff2463C2),
                                                width: 20,
                                              ),
                                            )
                                          : const SizedBox(
                                              width: 28,
                                            ),
                                      Text(data.tableName ?? ''),
                                    ],
                                  ),
                                ),
                              ),
                            );
                          });
                    }),
              ),
            )
          ],
        );
      }),
    );
  }
}
